@@ -31,6 +31,7 @@
 #include <dbus/dbus-glib-bindings.h>
 
 #include "cryptui.h"
+#include "cryptui-key-store.h"
 
 #include "seahorse-secure-memory.h"
 #include "seahorse-tool.h"
@@ -285,26 +286,45 @@ encrypt_sign_start (SeahorseToolMode *mode, const gchar *uri, gpgme_data_t urida
  * SIGN 
  */
 
+static gboolean 
+signer_filter (CryptUIKeyset *ckset, const gchar *key, gpointer user_data)
+{
+    guint flags = cryptui_keyset_key_flags (ckset, key);
+    return flags & CRYPTUI_FLAG_CAN_SIGN;
+}
+
 static gpgme_key_t
 prompt_signer ()
 {
     gpgme_error_t gerr = GPG_OK;
     CryptUIKeyset *keyset;
+    CryptUIKeyStore *ckstore;
     gpgme_key_t key = NULL;
     gpgme_ctx_t ctx = NULL;
     gchar *signer;
     gchar *id;
+    guint count;
+    GList *keys;
     
     keyset = cryptui_keyset_new ("openpgp", TRUE);
+    ckstore = cryptui_key_store_new (keyset, TRUE, NULL);
+    cryptui_key_store_set_filter (ckstore, signer_filter, NULL);
     
-    if (cryptui_keyset_get_count (keyset) == 0) {
+    count = cryptui_key_store_get_count (ckstore);
+    
+    if (count == 0) {
         cryptui_need_to_get_keys ();
         return NULL;
+    } else if (count == 1) {
+        keys = cryptui_key_store_get_all_keys (ckstore);
+        signer = (gchar*) keys->data;
+        g_list_free (keys);
+    } else {
+        signer = cryptui_prompt_signer (keyset, _("Choose Signer"));
     }
     
-    signer = cryptui_prompt_signer (keyset, _("Choose Signer"));
     if (signer) {
-        
+            
         id = cryptui_keyset_key_raw_keyid (keyset, signer);
         g_free (signer);
         
@@ -320,7 +340,8 @@ prompt_signer ()
         if (!GPG_IS_OK (gerr))
             seahorse_util_handle_gpgme (gerr, _("Couldn't load keys"));
     }
-
+    
+    g_object_unref (ckstore);
     g_object_unref (keyset);
     return key;
 }
@@ -469,7 +490,7 @@ import_show (SeahorseToolMode *mode)
 	t = g_strdup_printf (ngettext ("Imported %d key", "Imported %d keys", mode->imports), mode->imports);
         
     dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
-                                  GTK_MESSAGE_INFO, GTK_BUTTONS_OK, t);
+                                  GTK_MESSAGE_INFO, GTK_BUTTONS_OK, t, NULL);
     g_free (t);
     gtk_dialog_run (GTK_DIALOG (dlg));
     gtk_widget_destroy (dlg);
